#
# command.py - dtv2ser high level commands
#
# Written by
#  Christian Vogelgsang <chris@vogelgsang.org>
#
# This file is part of dtv2ser.
# See README for copyright notice.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#  02111-1307  USA.
#

from dtv2ser.status   import *
from dtv2ser.cmdline  import *
from dtv2ser.sercon   import *
from dtv2ser.transfer import *

class Command:
  """The high level commands of dtv2ser."""

  # client version: 0.2
  client_major = 0
  client_minor = 2

  def __init__(self,con):
    self.cmdline  = CmdLine(con)
    self.transfer = Transfer(con)

  # ----- version -----------------------------------------------------------

  def get_server_version(self):
    """Return version of dtv2ser firmware.
    Returns (status,major,minor)
    """
    result = self.cmdline.do_command('v')
    if result != STATUS_OK:
      return (result,0,0)
    (result,version) = self.cmdline.get_word()
    major = version >> 8
    minor = version & 0xff
    return (result,major,minor)

  def get_client_version(self):
    """Return version of client"""
    return (self.client_major,self.client_minor)

  def is_compatible_version(self,server_major,server_minor):
    """Check if host and server versions are compatible.
    Return True or False
    """
    # client is too old
    if self.client_major < server_major:
      return False
    elif self.client_major == server_major:
      # client has more current version
      return self.client_minor >= server_minor
    else:
      # client is newer
      return True

  # ----- dtv client commands -----------------------------------------------
  
  def read_memory(self,rom,start,length,callback=lambda x:True):
    """Read a memory block from the DTV.
    Return (result,data,client_rx_rate,server_rx_rate).
    """
    cmd = "r%02x%06x%06x" % (rom,start,length)
    (result,data) = self.transfer.do_receive_command(cmd,length,callback=callback);
    (crx,srx)     = self.transfer.get_rx_rates()
    return (result,data,crx,srx)

  def write_memory(self,rom,start,data,callback=lambda x:True):
    """Write a memory block to the DTV
    Return (result,client_tx_rate,server_tx_rate).
    """
    cmd = "w%02x%06x%06x" % (rom,start,len(data))
    result    = self.transfer.do_send_command(cmd,data,callback=callback)
    (ctx,stx) = self.transfer.get_tx_rates()
    return (result,ctx,stx)

  def write_boot_memory(self,start,data,callback=lambda x:True):
    """Write a boot memory block to the DTV
    Return (result,client_tx_rate,server_tx_rate).
    """
    cmd = "b%04x%04x" % (start,len(data))
    result    = self.transfer.do_send_command(cmd,data,callback=callback)
    (ctx,stx) = self.transfer.get_tx_rates()
    return (result,ctx,stx)

  def dtv_reset(self):
    """Reset the DTV and enter dtvtrans if dtvmon is available.
    Return result.
    """
    result = self.cmdline.do_command("x")
    if result != STATUS_OK:
      return result
    (result,val) = self.cmdline.get_byte()
    return result
  
  def go_memory(self,addr):
    """Go/jump to memory location.
    Return result.
    """
    cmd = "g%04x" % addr
    result = self.cmdline.do_command(cmd)
    if result != STATUS_OK:
      return result
    (result,val) = self.cmdline.get_byte()
    return result

  # ----- diagnose commands -------------------------------------------------

  def diagnose_read_memory_only_dtv(self,rom,start,length,callback=lambda x:True):
    """Read a memory block from the DTV but do not receive the data via serial.
    Return (result,crx,srx).
    """
    cmd = "ddr%02x%06x%06x" % (rom,start,length)
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,False,callback=callback)
    (crx,srx) = self.transfer.get_rx_rates()
    return (result,crx,srx)

  def diagnose_write_memory_only_dtv(self,rom,start,length,callback=lambda x:True,pattern=0):
    """Write a memory block to the DTV but do not send the data via serial.
    Return (result,crx,srx).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "ddw%02x%06x%06x" % (rom,start,length)
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,True,callback=callback)
    (ctx,stx) = self.transfer.get_tx_rates()
    return (result,ctx,stx)

  def diagnose_verify_memory_only_dtv(self,rom,start,length,callback=lambda x:True,pattern=0):
    """Read and verify a memory block fromo the DTV but do not send the data via serial.
    Return (result,crx,srx).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "ddv%02x%06x%06x" % (rom,start,length)
    result = self.transfer.do_diagnose_command_only_dtv(cmd,length,False,callback=callback)
    (ctx,stx) = self.transfer.get_rx_rates()
    return (result,ctx,stx)

  def diagnose_read_memory_only_client(self,length,callback=lambda x:True,pattern=0):
    """Read a memory block from the dtv2ser device but do not transfer from DTV.
    Return (result,crx,srx).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "dsr00000000%06x" % length
    result = self.transfer.do_diagnose_receive_command_only_client(cmd,length,callback=callback)
    (crx,srx) = self.transfer.get_rx_rates()
    return (result,crx,srx)
    
  def diagnose_write_memory_only_client(self,length,callback=lambda x:True,pattern=0):
    """Write a memory block to the dtv2ser device but do not transfer to the DTV.
    Return (result,crx,srx).
    """
    if pattern != 0:
      result = self.transfer.set_diagnose_pattern(pattern)
      if result != STATUS_OK:
        return result

    cmd = "dsw00000000%06x" % length
    result = self.transfer.do_diagnose_send_command_only_client(cmd,length,callback=callback)
    (ctx,stx) = self.transfer.get_tx_rates()
    return (result,ctx,stx)

  def diagnose_get_transfer_result(self):
    """Return the last transfer result.
    Return result
    """
    return self.transfer.get_transfer_result()

  def diagnose_do_command(self,cmd):
    """Issue a command directly.
    Return result
    """
    return self.cmdline.do_command(cmd)

  # ----- high level commands -----------------------------------------------

  def read_basic_size(self):
    """Read the basic start and end from DTV
    
    Return (result,basis_start,basic_end)
    """
    (result,data,cr,sr) = self.read_memory(0,0x2b,4)
    if result != 0:
      return (result,0,0)
    basic_start = ord(data[0]) + 256 * ord(data[1])
    basic_end   = ord(data[2]) + 256 * ord(data[3])
    if basic_end < basic_start:
      return (CLIENT_ERROR_INVALID_BASIC_RANGE,0,0)
    return (result,basic_start,basic_end)
  
