#
# sercon.py - serial low level connection
#
# Written by
#  Christian Vogelgsang <chris@vogelgsang.org>
#
# This file is part of dtv2ser.
# See README for copyright notice.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#  02111-1307  USA.
#

# requires PySerial from http://pyserial.sourceforge.net !

import serial
import time
import sys

from dtv2ser.status import *

class SerCon:
  """Encapsulates a low level serial connection to a dtv2ser server device."""

  # time out to wait for server become ready
  ready_timeout = 50
  
  def __init__(self, serial_port, serial_baud, serial_timeout=2):
    """Open a serial connection to the dtv2ser server."""
    try:
      self.ser = serial.Serial(port=serial_port,
                              baudrate=serial_baud,
                              bytesize=serial.EIGHTBITS,
                              parity=serial.PARITY_NONE,
                              stopbits=serial.STOPBITS_ONE,
                              timeout=serial_timeout,
                              writeTimeout=serial_timeout,
                              xonxoff=0,
                              rtscts=1)
      self.ser.open()
      if self.ser.isOpen():
        self.ser.flushInput()
        self.ser.flushOutput()
        self.valid = True
      else:
        self.valid = False
    except:
      self.valid = False
      
  def __del__(self):
    """Close connection to dtv2 server."""
    if self.valid and self.ser.isOpen():
      self.ser.close()
      
  def is_connected(self):
    """Is client connected?
    Returns True if the server is ready, otherwise False.
    """
    return self.valid and self.ser.isOpen()
    
  def wait_for_server(self,timeout=0):
    """Wait for server to become ready.
    Returns result code.
    """
    if timeout == 0:
      timeout = self.ready_timeout
    for t in xrange(timeout):
      if self.ser.getCTS():
        return STATUS_OK
      time.sleep(0.1)
    return CLIENT_ERROR_SERVER_TIMEOUT

  # ----- basic I/O -----
  
  def receive_data(self,length):
    """Read n bytes from server.
    Returns (result,data).
    """
    result = self.ser.read(length)
    if len(result) != length:
      return (CLIENT_ERROR_SERVER_TIMEOUT,result)
    return (STATUS_OK,result)

  def send_data(self,data):
    """Write n bytes to server.
    Returns result.
    """
    try:
      self.ser.write(data)
    except serial.SerialException:
      print "TIMEOUT!"
      return CLIENT_ERROR_SERVER_TIMEOUT
    return STATUS_OK

  # ----- block transfers -----

  def dump_cts(self,num):
    for a in xrange(num):
      isCTS = self.ser.getCTS()
      if isCTS:
        print "*",
      else:
        print "-",
      sys.stdout.flush()
      time.sleep(0.1)
    print

  def block_start(self):
    """Handle the start of a block.
    Returns (status,block_len)
    """
    # start of block: get block length (u16)
    (status,raw) = self.receive_data(2)
    if status!=STATUS_OK:
      return status,0
    
    # convert binary length to word
    try:
      block_len = ord(raw[0]) * 256 + ord(raw[1])
    except:
      return (CLIENT_ERROR_INVALID_HEX_NUMBER,0)

    #print "begin block: %04x" % block_len
    return (status,block_len)

  def calc_crc16(self,data):
    """Calc a block checksum with crc16.
    Return crc16.
    """
    crc = 0xffff
    for a in data:
      b = ord(a)
      crc ^= b;
      for i in xrange(8):
        if crc & 1:
          crc = (crc >> 1) ^ 0xA001
        else:
          crc = (crc >> 1)
    return crc & 0xffff
    
  def block_end(self,data):
    """Handle the end of a block.
    Returns status
    """
    # get result: crc16 (u16) + status
    (status,raw) = self.receive_data(3)
    if status!=STATUS_OK:
      return status

    # convert block length
    try:
      crc16  = ord(raw[0]) * 256 + ord(raw[1])
    except:
      return CLIENT_ERROR_INVALID_HEX_NUMBER
    
    status = ord(raw[2])
    #print "end block: %04x %02d" % (crc16,status)

    # check client status
    if status!=STATUS_OK:
      return status | TRANSFER_ERROR_MASK
    
    # check crc16  
    my_crc16 = self.calc_crc16(data)
    if crc16!=my_crc16:
      #print "my crc: %04x, server crc: %04x" % (my_crc16,crc16)
      status = CLIENT_ERROR_CRC16_MISMATCH

    return status;

  def receive_block(self,length,block_size,callback=lambda x:True):
    """Send a large block of data in blocks given by block_size.
    Use optional callback to get feedback while transfer.
    Returns (result,duration,data).
    """
    status = STATUS_OK
    pos = 0
    data = ''
    
    # begin upload
    start_time = time.time()

    # transfer blocks
    while length > 0:
      callback(pos)

      # block begin
      status,block_len = self.block_start()
      if status!=STATUS_OK:
        break
    
      # now receive block
      status,raw = self.receive_data(block_len)
      if status!=STATUS_OK:
        break
      data += raw

      # block end
      status = self.block_end(raw)
      if status!=STATUS_OK:
        break
      
      pos += block_len
      length -= block_len
      
    # end upload
    end_time = time.time()
          
    # all ok
    return (status,end_time - start_time,data)

  def send_block(self,data,block_size,callback=lambda x:True):
    """Send a large block of data in blocks given by block_size.
    Use optional callback to get feedback while transfer.
    Returns (result,duration).
    """
    status = STATUS_OK
    length = len(data)
    pos = 0
    
    # begin upload
    start_time = time.time()

    # transfer blocks
    while length > 0:
      callback(pos)

      # block begin
      (status,block_len) = self.block_start()
      if status!=STATUS_OK:
        break
    
      # now send block
      raw = data[pos:pos+block_len]
      status = self.send_data(raw)
      if status!=STATUS_OK:
        break

      # block end
      status = self.block_end(raw)
      if status!=STATUS_OK:
        break
      
      pos += block_len
      length -= block_len
      
    # end upload
    end_time = time.time()
          
    # all ok
    return (status,end_time - start_time)
